/*
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package io.blitz.maven;

import io.blitz.curl.AbstractTest;
import io.blitz.curl.config.BasicAuthentication;
import io.blitz.curl.config.Content;
import io.blitz.curl.config.HttpHeader;
import io.blitz.curl.config.variable.AlphaVariable;
import io.blitz.curl.config.variable.IVariable;
import io.blitz.curl.config.variable.ListVariable;
import io.blitz.curl.config.variable.NumberVariable;
import io.blitz.curl.config.variable.UdidVariable;
import java.net.HttpCookie;
import org.apache.maven.plugin.AbstractMojo;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Superclass for all blitz.io curl tests goals.
 */
public abstract class BlitzMojo extends AbstractMojo {

    /**
     * The test target (must be acessible online).
     * @parameter expression="${test.url}"
     * @required
     */
    protected URL url;
    
    /**
     * Region from which the test should start
     * @parameter expression="${test.region}"
     */
    protected String region;

    /**
     * User agent to be used on the test. It is serialized with the name 'user-agent'
     * @parameter
     */
    protected String userAgent;
    
    /**
     * referrer to be used on the test
     * @parameter
     */
    protected URL referrer;
    
    /**
     * The expected status response
     * @parameter
     */
    protected Integer status;
    
    /**
     * The timeout to be used on the test
     * @parameter
     */
    protected Integer timeout;
    
    /**
     * Describes the ssl for the test
     * @parameter
     */
    protected String ssl;
    
    /**
     * Cookies to send on the test
     * @parameter
     */
    protected List<String> cookies;
    
    /**
     * headers to send on the test
     * @parameter
     */
    protected List<String> headers;
    
    /**
     * User and pasword for basic authentication to be used on the test
     * @parameter
     */
    protected String testAuthentication;
    
    /**
     * Content data to be sent on the test
     * @parameter
     */
    protected List<String> data;
    
    /**
     * Variables to be used on the url. The key should be the variable name and
     * the variable instance
     * @parameter
     */
    protected List<Variable> variables;
    
    /**
     * Your blitz.io username (usually an email address)
     * @parameter expression="${blitz.username}"
     * @required
     */
    protected String username;
    
    /**
     * You blitz.io authentication key. You can find your api key by typing
     * --api-key into the blitz play bar.
     * @parameter expression="${blitz.apiKey}"
     * @required
     */
    protected String apiKey;
    
    /**
     * Configures the base test parameters. Should be called in the subclass
     * execute method.
     * @param test 
     */
    protected void configure(AbstractTest test) {
        test.setUrl(url);
        
        if(region!=null)
            test.setRegion(region);
        if(userAgent!=null)
            test.setUserAgent(userAgent);
        if(referrer!=null)
            test.setReferrer(referrer);
        if(status!=null)
            test.setStatus(status);
        if(timeout!=null)
            test.setTimeout(timeout);
        if(ssl!=null)
            test.setSsl(ssl);
        if(data!=null && !data.isEmpty())
            test.setContent(new Content(data));
        Collection<HttpCookie> cookieList = listCookies(cookies);
        if(!cookieList.isEmpty())
            test.setCookies(cookieList);
        Collection<HttpHeader> headerList = listHeaders(headers);
        if(!headerList.isEmpty())
            test.setHeaders(headerList);
        if(testAuthentication!=null) {
            String[] auth = testAuthentication.split(":");
            if(auth.length == 2)
                test.setUser(new BasicAuthentication(auth[0].trim(), auth[1].trim()));
        }
        if(variables!= null && !variables.isEmpty()) {
            Map<String, IVariable> map = mapVariables(variables);
            if(!map.isEmpty())
                test.setVariables(map);
        }
            
    }
    
    /**
     * Builds a list of <code>HttpCookie</code> from a list of strings.
     * @param cookies list of cookies represented as strings
     * @return list of <code>HttpCookie</code>
     */
    protected Collection<HttpCookie> listCookies(Collection<String> cookies) {
        Collection<HttpCookie> list = new ArrayList<HttpCookie>();
        if(cookies != null) {
            for(String cookieStr : cookies) {
                String[] array = cookieStr.split("=");
                //only use valid cookie strings
                if(array.length == 2) {
                    list.add(new HttpCookie(array[0].trim(), array[1].trim()));
                }
            }
        }
        return list;
    }

    /**
     * Builds a list of <code>HttpHeader</code> from a list of strings.
     * @param headers list of headers represented as strings
     * @return list of <code>HttpHeader</code>
     */
    protected Collection<HttpHeader> listHeaders(Collection<String> headers) {
        Collection<HttpHeader> list = new ArrayList<HttpHeader>();
        if(headers != null) {
            for(String headerStr : headers) {
                String[] array = headerStr.split(":");
                //only use valid cookie strings
                if(array.length == 2) {
                    list.add(new HttpHeader(array[0].trim(), array[1].trim()));
                }
            }
        }
        return list;
    }
    
    /**
     * Creates a IVariable map from from the list given in the configuration
     * @param collection list of configuration variables
     * @return map of IVariables
     */
    protected Map<String, IVariable> mapVariables(List<Variable> collection) {
        Map<String, IVariable> map = new HashMap<String, IVariable>();
        if(collection!= null) {
            for(Variable var : collection) {
                if(var.getName() != null && var.getName().length() > 0 &&
                        var.getType() != null) {
                    
                    if("list".equals(var.getType())) {
                        map.put(var.getName(), new ListVariable(var.getItems()));
                    }
                    else if("alpha".equals(var.getType())) {
                        map.put(var.getName(), 
                                new AlphaVariable(var.getMin(), var.getMax()));
                    }
                    else if("number".equals(var.getType())) {
                        map.put(var.getName(), 
                                new NumberVariable(var.getMin(), var.getMax()));
                    }
                    else if("udid".equals(var.getType())) {
                        map.put(var.getName(), new UdidVariable());
                    }
                }
            }
        }
        return map;
    }
}
