/*
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package io.blitz.maven;

import io.blitz.curl.Rush;
import io.blitz.curl.config.Interval;
import io.blitz.curl.config.Pattern;
import io.blitz.curl.exception.BlitzException;
import io.blitz.curl.rush.IRushListener;
import io.blitz.curl.rush.Point;
import io.blitz.curl.rush.RushResult;
import org.apache.maven.plugin.MojoExecutionException;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * Goal to run a http://blitz.io Rush load test.
 *
 * @goal rush
 * 
 * @phase integration-test
 * 
 * @requiresOnline true
 */
public class RushMojo extends BlitzMojo {


    /**
     * List of intervals required for rush test.
     * @parameter
     * @required
     */
    private List<Interval> pattern;
    
    /**
     * Execute the rush mojo test.
     * @throws MojoExecutionException if the test fails or an error occur
     */
    public void execute() throws MojoExecutionException {
        
        Rush rush = new Rush(username, apiKey);
        rush.addListener(new IRushListener() {
            public boolean onData(RushResult result) {
                if(!result.getTimeline().isEmpty()) {
                    List<Point> list = ((List<Point>)result.getTimeline());
                    Point last = list.get(result.getTimeline().size() - 1);
                    String template =  "[{0}] {1}ms response time with {2} users"
                        + " (total: {3}, hits: {4}, errors: {5}, timeouts:{6})";
                    SimpleDateFormat sdf = new SimpleDateFormat("mm:ss");
                    String timestamp = sdf.format(last.getTimestamp());
                    long ms = Math.round(last.getDuration() * 1000);
                    String message = MessageFormat.format(template, timestamp, 
                            ms, last.getVolume(), last.getTotal(), last.getHits(), 
                            last.getErrors(), last.getTimeouts());
                    getLog().info(message);
                }
                return true;
            }
        });
        getLog().debug("Configuring test.");
        configure(rush);
        if(pattern != null && !pattern.isEmpty())
            rush.setPattern(new Pattern(pattern));
        getLog().debug("Starting test.");
        try {
            rush.execute();
            getLog().debug("Rush successfully executed.");
        }
        catch(BlitzException ex) {
            getLog().error(ex);
            String message = MessageFormat
                    .format("{0}: {1}", ex.getError(), ex.getReason());
            throw new MojoExecutionException(message);
        }
    }
}
